
#define _USE_VInv
#define _USE_SHADOWS
#define _USE_WVP
#define _USE_W
#define _USE_CLOAKING
#define _USE_WTrans
#define _USE_SPECMAP
#define _USE_DIFFUSEMAP
#define _USE_NORMALMAP
#define _USE_MATERIAL
#define _USE_BACKBUFFER
#define _USE_LIGHTS


// the implementation is in this file
#include "StandardParameters.fxh"
#include "Effect_common.fxh"
#include "Shadows.fxh"

float3 g_cDistortionRimColor : Specular
<
	string UIName = "distortion rim color";
    string UIWidget = "Ambient Light Color";
    string Space = "material";
    
> = {0.25f, 0.25f, 0.25f};

// The current file just makes it easy for the game to find and validate specific implementations
// this very files should contain all variants and fallback for an object with
// a diffuse map, a normal map and a specular map.

struct diffuseNormalSpecularVertexOutput {
    float4 pPosition		: POSITION;
    float2 tcDiffuse		: TEXCOORD0;
    float2 tcNormal			: TEXCOORD1;
    float2 tcSpecular		: TEXCOORD2;
    float4 tcShadowMap		: TEXCOORD3;
    float4 vLight			: COLOR0;
    float4 vHalfAngle		: COLOR1;
};


///////////////////////////////////////////////////////////////////////////////

diffuseNormalSpecularVertexOutput VS_DistortDiffuse(vertexInput_ST IN,
												  	uniform float4 vLightPos,
												  	uniform float4 vLightColor,
												  	uniform bool bDoShadows)
{
	diffuseNormalSpecularVertexOutput OUT;
   
	// Transform position to clip space and output it
    OUT.pPosition = ModelToScreenSpace(IN.pPosition);
    
    // Output texture coordinates.
	OUT.tcDiffuse  = IN.tcDiffuse;
	OUT.tcNormal   = IN.tcDiffuse;
	OUT.tcSpecular = IN.tcDiffuse;
	
	if (bDoShadows)
	{	
		// get world position for diffuse and specular lighting
		float4 pWorldPosition = ModelToWorldSpace(IN.pPosition);
		
		// generate shadow space pos
		OUT.tcShadowMap = WorldSpaceToShadowSpace ( pWorldPosition );
		// make z linear
		OUT.tcShadowMap.z/=DEPTH_SCALE;
		OUT.tcShadowMap.w/=DEPTH_SCALE;
	}
	else
	{
		OUT.tcShadowMap = 0;
	}
	
	// distort the model in screen space - jmg
	OUT.pPosition.xy += ComputeCloakingDistortion(OUT.pPosition.xy, g_fDistortionWaveAmplitude, g_CloakTime);
	
	// get world position for diffuse and specular lighting
	float4 pWorldPosition = ModelToWorldSpace(IN.pPosition);
	
	// light intensity and direction from pos and light pos
	float4 vLight = ComputeLightDirAndIntensityToWorldPoint(vLightPos.xyz , pWorldPosition.xyz, vLightPos.w, vLightColor.w);
		
	// Put the unpacked normal space light vector in diffuse 0
	// put intensity in w
	float3 vNormalSpaceLight = ToNormalSpace(WorldToModelSpaceRotate(vLight.xyz) , IN.vNormal, IN.vTangent, IN.vBinormal);
	OUT.vLight = float4(vNormalSpaceLight, vLight.w);

	// Put the half-angle vector in diffuse 1
	float3 vNormalSpaceHalfAngle = ToNormalSpace(WorldToModelSpaceRotate(ComputeHalfAngle(vLight.xyz , pWorldPosition)), IN.vNormal, IN.vTangent, IN.vBinormal);
	OUT.vHalfAngle = float4(vNormalSpaceHalfAngle, 1);

	return OUT;
}

///////////////////////////////////////////////////////////////////////////////
float4 PS_DistortDiffuse(diffuseNormalSpecularVertexOutput IN,
						 uniform float3 cLightColor,
						 uniform float4 cMaterialDiffuse, // we want alpha mat value to carry through
						 uniform float3 cMaterialAmbient,
						 uniform bool bDoShadows): COLOR
{
	// fetch base texture
	float4 cDiffTex =
		tex2D(g_sampDiffuse, IN.tcDiffuse);
	// fetch normal texture
	float4 vNormal =
		tex2D(g_sampNormal, IN.tcNormal);
	// fetch specular texture
	float4 cSpecTex =
		tex2D(g_sampSpecular, IN.tcSpecular);

	// Unpack our vector.
	vNormal.xyz = Unpack(vNormal.xyz);
	float4 vLight = IN.vLight;
	float3 vHalfAngle = IN.vHalfAngle.xyz;

	// N.L
	float fNDotL = vLight.w * saturate(dot(vNormal.xyz, vLight.xyz));
	// N.H
	float fNDotH = vLight.w * saturate(dot(vNormal.xyz, vHalfAngle.xyz));
	if(fNDotL <= 0)
	{
		fNDotH = 0;
	}
	else
	{
		if (bDoShadows)
		{
			// do 4 sample PCF
			float shadow = ComputeShadow4PCF(IN.tcShadowMap);
	    	fNDotL *= shadow;
	    	fNDotH *= shadow;
	    }
    }
	
	// We use specMag^(shininess) for our fall-off.
	float fNDotHExp = pow(fNDotH, shininess);
	
	// Compute the specular color
	// use white light color, actually looks better
	float3 cSpecular = g_cMaterialSpecular.xyz * cSpecTex.xyz * cLightColor * fNDotHExp;
	
	// Compute the diffuse color, witht the per pass light color parameter
	float3 cDiffuse = cMaterialDiffuse.rgb * cLightColor * fNDotL;
	
	// Compute the ambient color, we do not use light ambient so it ends up 
	// being only material ambient, default black.
	float3 cAmbient = cMaterialAmbient.xyz;
		
	// combine lighting with diffuse and detail texture
	return float4(saturate(cAmbient+cDiffuse) * cDiffTex.rgb + cSpecular, cMaterialDiffuse.a * g_fOpacity);
}

///////////////////////////////////////////////////////////////////////////////

struct screenDistortionVertexOutput {
    float4 pPosition		: POSITION;
    float2 tcDiffuse		: TEXCOORD0;
    float2 tcNormal			: TEXCOORD1;
    float2 tcSpecular		: TEXCOORD2;
    float3 tcIncident		: TEXCOORD3;
};

///////////////////////////////////////////////////////////////////////////////

screenDistortionVertexOutput VS_DistortBackground(vertexInput_ST IN,
												  uniform float4 vLightPos,
												  uniform float4 vLightColor,
												  uniform float3 vCameraPos)
{
	screenDistortionVertexOutput OUT;
	float4 vPositionModel;
	float3 vCameraModel;
	float3 vIncidentModel;

	// transform the vertex and camera position from world space to model space
	vPositionModel 	 = float4(IN.pPosition.xyz, 1.0f);
    vCameraModel.xyz = vCameraPos.xyz - g_mWorld[3];
    
    // compute the camera-to-vertex vector in normal space and pack it
    vIncidentModel = vPositionModel - vCameraModel;
    vIncidentModel = WorldToModelSpaceRotate(vIncidentModel);
    vIncidentModel = normalize(vIncidentModel);
    OUT.tcIncident = float4(ToNormalSpace(vIncidentModel, IN.vNormal, IN.vTangent, 
    									  IN.vBinormal), 0.0f);
    
    // transform the vertex position from model to screen space
    OUT.pPosition = ModelToScreenSpace(IN.pPosition);
    
    // compute the diffuse color texture coordinates in screen space
    OUT.tcDiffuse.xy = ((OUT.pPosition.xy / OUT.pPosition.ww) + 1.0f) * 0.5f;
    OUT.tcDiffuse.y  = 1.0f - OUT.tcDiffuse.y;
    
    // output the texture coordinates for the diffuse, normal and specular maps
	OUT.tcNormal 	 = IN.tcDiffuse;
	OUT.tcSpecular 	 = IN.tcDiffuse;

	// distort the model in screen space - jmg
	OUT.pPosition.xy += ComputeCloakingDistortion(OUT.pPosition.xy, g_fDistortionWaveAmplitude, g_CloakTime);

	return OUT;
}

///////////////////////////////////////////////////////////////////////////////

float4 PS_DistortBackground(screenDistortionVertexOutput IN,
							uniform float3 cLightColor,
							uniform float4 cMaterialDiffuse, // we want alpha mat value to carry through
							uniform float3 cMaterialAmbient): COLOR
{
	float4 vNormal;
	float4 vBackBufferColor;
	float3 vFinalColor;
	float3 vIncident;
	float  fresnelFactor;
	
	// fetch the packed normal from the normal map and unpack it accordingly
	vNormal = tex2D(g_sampNormal, IN.tcNormal);
	vNormal = Unpack(vNormal);
		
	// normalize the unpacked normal and incident vectors
	vNormal   = normalize(vNormal);
	vIncident = normalize(IN.tcIncident.xyz);
	
	// compute the fresnel coefficient
    fresnelFactor = pow(1.0f + dot(vIncident.xyz, vNormal.xyz), 3.0f);

	// if the cloak color is anything but black, g_cCloakColor.a will be 1.0f; this indicates
	// whether the "outline" of the ship should be visible; if the outline should not be visible
	// and the diffuse of the ship has faded away to nothing, the fresnelFactor will be 0.0f - jmg   
    fresnelFactor *= g_cCloakColor.a;
    
   	// sample the back buffer
	vBackBufferColor = tex2D(g_sampBackBuffer, IN.tcDiffuse.xy + fresnelFactor * 0.1f);
	
	// prevent the final color from being completely washed out by the cloak color
	fresnelFactor = min(fresnelFactor, 0.7f);
	
	// blend between the cloak color and the back buffer color; the fresnel coefficient
	// will cause more distortion around the edges of the model
	vFinalColor.rgb = lerp(vBackBufferColor.rgb, g_cCloakColor, fresnelFactor);
	    
	// return the final color
	return float4(vFinalColor.rgb, g_cCloakColor.a);
}

///////////////////////////////////////////////////////////////////////////////

technique cloaking
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1ST";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_DEFAULT;
// the number of lights the effect can do
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
	// this pass only renders the model to the z-buffer and is necessary to prevent
	// artifacts when the model becomes transparent; this is due to those faces of the
	// model with are marked as camera facing by the rasterizer (due to their winding 
	// order) but are occluded by the model itself
	pass ZPass
	{
		AlphaBlendEnable 	= false;
		AlphaTestEnable 	= false;
		ZFunc 				= LESSEQUAL;
		ZEnable 			= true;
		ZWriteEnable 		= true;
		SrcBlend 			= ZERO;
		DestBlend 			= ONE;
		VertexShader 		= compile vs_3_0 VS_DistortDiffuse(g_pBasePassLightPosition, g_cBasePassLightColor, false);
		PixelShader  		= compile ps_3_0 PS_DistortDiffuse(g_cBasePassLightColor, g_cMaterialDiffuse, g_cMaterialAmbient, false);
	}
        
    // this pass uses the frame buffer as a texture and distorts the model in screen
    // space to achieve a transition between being cloaked and uncloaked
    pass DistortPass
    {
    	AlphaBlendEnable 	= true;
		ZFunc 				= LESSEQUAL;
		ZEnable 			= true;
		ZWriteEnable 		= false;
		SrcBlend 			= SRCALPHA;
		DestBlend 			= INVSRCALPHA;
		VertexShader 		= compile vs_3_0 VS_DistortBackground(g_pBasePassLightPosition, g_cBasePassLightColor, g_pEye);
		PixelShader  		= compile ps_3_0 PS_DistortBackground(g_cBasePassLightColor, g_cMaterialDiffuse, g_cMaterialAmbient);
    }
    pass ShadowPass
    <
    // this annotation tells the Mad3D_Renderer that this is the shadow pass to use only once
    bool bShadowPass = true;
    >
    {
		VertexShader 		= compile vs_3_0 VS_DistortDiffuse(g_pBasePassLightPosition, g_cBasePassLightColor, true);
		PixelShader  		= compile ps_3_0 PS_DistortDiffuse(g_cBasePassLightColor, g_cMaterialDiffuse, g_cMaterialAmbient, true);
    }
    // this pass renders the color channels of the model
    pass DiffusePass
    <
    // this annotation tells the Mad3D_Renderer that this is the base pass to use only once
    bool bBasePass = true;
    >
    {
		VertexShader 		= compile vs_3_0 VS_DistortDiffuse(g_pBasePassLightPosition, g_cBasePassLightColor, false);
		PixelShader  		= compile ps_3_0 PS_DistortDiffuse(g_cBasePassLightColor, g_cMaterialDiffuse, g_cMaterialAmbient, false);
    }
    
    pass PerLightPass
    <
    // this annotation tells the Mad3D_Renderer that this pass can be called 
    // once per additionnal light after the base pass
    // this is to circumvent the lack of constant register space in P_1.1
	bool bPerLightPass = true;    
    >
    {
    	AlphaBlendEnable 	= true;
		DestBlend 			= one;
		VertexShader 		= compile vs_3_0 VS_DistortDiffuse(g_pPerLightPassLightPosition, g_cPerLightPassLightColor, false);
		PixelShader  		= compile ps_3_0 PS_DistortDiffuse(g_cPerLightPassLightColor, g_cMaterialDiffuse, float3(0,0,0), false);
    }
}
// do nothing on fallback
technique RenderFallback
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_LASTRESORT;
// the number of lights the effect can do
// this effect ignores lights so it can render any number ...
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
}

///////////////////////////////////////////////////////////////////////////////

// Default lighting.
#include "FixedPipelineDiffuse.fxh"

///////////////////////////////////////////////////////////////////////////////
