///////////////////////////////////////////////////////////////////////////////
// Vertex Shader
///////////////////////////////////////////////////////////////////////////////
#define _USE_WVP
#define _USE_W
#define _USE_CLOAKING
#define _USE_CLIPPLANE


#include "StandardParameters.fxh"
#include "effect_common.fxh"

#ifdef _WIN32
texture g_texDamageNoise  : Mad3D_VolumeTexture;
sampler3D g_sampDamageNoise = sampler_state 
{
    texture = <g_texDamageNoise>;
    AddressU  = CLAMP;
    AddressV  = CLAMP;
    AddressW  = CLAMP;
    MIPFILTER = POINT;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};
#else
// on Xbox sampler have a harder time
sampler3D g_sampDamageNoise : Mad3D_VolumeTexture 
= sampler_state 
{
    AddressU  = CLAMP;
    AddressV  = CLAMP;
    AddressW  = CLAMP;
    MIPFILTER = POINT;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};
#endif


///////////////////////////////////////////////////////////////////////////////
// this value must exactly match the enum value of
// The output of the Vertex Shader
struct VS_OUTPUT
{
  // The transformed position
  float4 vPosition 		: POSITION;
  // The UV coordinates of the texture.
  float3 tcDamage 		: TEXCOORD0;  
  float3 vWorldPosition	: TEXCOORD1;
};

///////////////////////////////////////////////////////////////////////////////


VS_OUTPUT VS_Streaks(
  const float3 vPosition : POSITION,
  const float3 tcDiffuse : TEXCOORD0  )
{
	VS_OUTPUT vsOut;
	vsOut.vPosition = ModelToScreenSpace( vPosition );
	vsOut.vWorldPosition = ModelToWorldSpace(vPosition);
	
	// if cloaking is enabled, this object needs to be distorted so that its depth value is 
    // written correctly into the depth buffer; the cloaking effect expects the depth buffer 
    // to be established before it begins its processing - jmg
    if (g_bEnableCloakDistortion)
    {
	    // distort the model in screen space - jmg
		vsOut.vPosition.xy += ComputeCloakingDistortion(vsOut.vPosition.xy, g_fDistortionWaveAmplitude, g_CloakTime);
    }    
	
	vsOut.tcDamage = tcDiffuse;
	return vsOut;
}


///////////////////////////////////////////////////////////////////////////////
// Pixel Shader
///////////////////////////////////////////////////////////////////////////////

float4 PS_Streaks( VS_OUTPUT psIn ): COLOR0
{
	float a = 0.0f;

	// find dist to damage segment
	// display a cube around the UVs.
	if( psIn.tcDamage.x > 0 && psIn.tcDamage.x < 1
		&& psIn.tcDamage.y > 0 && psIn.tcDamage.y < 1
		&& psIn.tcDamage.z > 0 && psIn.tcDamage.z < 1)
	{
		a = tex3D( g_sampDamageNoise, psIn.tcDamage ).a;
		
		// if the pixel falls behind the specified clipping plane, kill it
		clip(dot(psIn.vWorldPosition - g_vClipPlanePosition, g_vClipPlaneNormal));
	}
	
	return float4(0, 0, 0, a);
}

///////////////////////////////////////////////////////////////////////////////

// renders the object with just a diffuse texture
technique RenderDiffuse
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosUVW";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_LOW;
// the number of lights the effect can do
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
	// render the diffuse texture
	pass DiffusePass1
	{
		AlphaBlendEnable 	= true;
		ZFunc 				= LESSEQUAL;
		ZEnable 			= true;
		ZWriteEnable 		= false;
		SrcBlend 			= SRCALPHA;
		DestBlend 			= INVSRCALPHA;
		VertexShader 		= compile vs_3_0 VS_Streaks();
		PixelShader 		= compile ps_3_0 PS_Streaks();
	}
}

// do nothing if 3D textures are not supported
technique RenderFallback
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosUVW";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_LASTRESORT;
// the number of lights the effect can do
// this effect ignores lights so it can render any number ...
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
}
