///////////////////////////////////////////////////////////////////////////////
// Vertex Shader
///////////////////////////////////////////////////////////////////////////////

#define _USE_ELAPSEDTIME
#define _USE_WVP
#define _USE_CLOAKING
#define _USE_W
#define _USE_MATERIAL
#define _USE_WInvTrans
#define _USE_LIGHTS
#define _USE_DIFFUSEMAP
#define _USE_NORMALMAP
#define _USE_SPECMAP
#define _USE_DETAILMAP
#define _USE_DECAL_TILING

#include "StandardParameters.fxh"
#include "Effect_common.fxh"



// The output of the Vertex Shader
struct VS_OUTPUT
{
  // The transformed position
  float4 vPosition : POSITION;
 // The UV coordinates of the texture.
  float2 vTextureUV : TEXCOORD0;
  // offset to make glow not beat in phase
  float3 fPhazeOffset : TEXCOORD1;
  
};
struct VS_OUTPUT_LIT
{
  // The transformed position
  float4 vPosition : POSITION;
  // The UV coordinates of the texture.
  float2 vTextureUV : TEXCOORD0;
   // diffuse
  float4 cDiffuse : COLOR0;
  // offset to make glow not beat in phase
  float3 fPhazeOffset : TEXCOORD1;
};
// scale on time
static float g_fPulseSpeed1 = 1.7;
static float g_fPulseSpeed2 = 0.7;
static float g_fPulseScale1 = 0.4;
static float g_fPulseScale2 = 0.9;
static float g_fPulseOffset = 0.3;

float MakePhaseSeed( float2 vUV , float3 vPos, float fPulseSpeed, float fPulseScale )
{
	// use pos as random seed (not very random really)
	// creates spacial coherency
	float3 RandomSeed = normalize( vPos );
	RandomSeed *= 3.14;
	vUV = vUV* 3.14;
	return fPulseScale*(1+sin( g_fElapsedTime*fPulseSpeed+RandomSeed.x+RandomSeed.y+RandomSeed.z + vUV.x+vUV.y ) );
}


///////////////////////////////////////////////////////////////////////////////
// simplest vertex shader just transforms and ignores normals
VS_OUTPUT VS_Fire1(
  const float3 vPosition : POSITION,
  const float3 vNormal : NORMAL,
  const float2 vTextureUV : TEXCOORD0
  )
{
	// Transform the position from object space to homogeneous projection space
	VS_OUTPUT vsOut;
	vsOut.vPosition = ModelToScreenSpace( vPosition );
	vsOut.vTextureUV = vTextureUV;
	vsOut.fPhazeOffset = MakePhaseSeed( vTextureUV, vPosition, g_fPulseSpeed1,g_fPulseScale1 );
	
	// if cloaking is enabled, this object needs to be distorted so that its depth value is 
    // written correctly into the depth buffer; the cloaking effect expects the depth buffer 
    // to be established before it begins its processing - jmg
    if (g_bEnableCloakDistortion)
    {
    	// distort the model in screen space - jmg
		vsOut.vPosition.xy += ComputeCloakingDistortion(vsOut.vPosition.xy, g_fDistortionWaveAmplitude, g_CloakTime);
	}
	
	return vsOut;
}
///////////////////////////////////////////////////////////////////////////////
// simplest vertex shader just transforms and ignores normals
VS_OUTPUT VS_Fire2(
  const float3 vPosition : POSITION,
  const float3 vNormal : NORMAL,
  const float2 vTextureUV : TEXCOORD0
  )
{
	// Transform the position from object space to homogeneous projection space
	VS_OUTPUT vsOut;
	vsOut.vPosition = ModelToScreenSpace( vPosition );
		
	vsOut.vTextureUV = vTextureUV*float2( 0.3f, 0.6f );
	vsOut.fPhazeOffset = g_fPulseOffset + MakePhaseSeed( vTextureUV, float3(vTextureUV.y,vTextureUV.y,vPosition.z), g_fPulseSpeed2, g_fPulseScale2);
	
	// if cloaking is enabled, this object needs to be distorted so that its depth value is 
    // written correctly into the depth buffer; the cloaking effect expects the depth buffer 
    // to be established before it begins its processing - jmg
    if (g_bEnableCloakDistortion)
    {
    	// distort the model in screen space - jmg
		vsOut.vPosition.xy += ComputeCloakingDistortion(vsOut.vPosition.xy, g_fDistortionWaveAmplitude, g_CloakTime);
	}
	
	return vsOut;
}
///////////////////////////////////////////////////////////////////////////////
// passes a diffuse color withthe two most dominant light on down
VS_OUTPUT_LIT VS_TransformLitTex1(
  const float3 vPosition : POSITION,
  const float3 vNormal : NORMAL,
  const float2 vTextureUV : TEXCOORD0
  )
{
	// Transform the position from object space to homogeneous projection space
	VS_OUTPUT_LIT vsOut;
	vsOut.vPosition = ModelToScreenSpace( vPosition );
	vsOut.vTextureUV = vTextureUV;
	// get world position for diffuse lighting
	float4 pWorldPosition = ModelToWorldSpace( vPosition );

	// start at black then add the lights
	float3 cDiffuseColor = g_cMaterialAmbient;

	// bring normal to world
	float3 vWorldNormal = normalize( ModelToWorldSpaceRotate( vNormal ) );
	
	// for each light just compute color
	// we only do the two most dominat light on decals
	for( int i = 0; i <2; i++ ) 
	{
		// light intensity and direction from vert pos and light pos
		float4 vLight = ComputeLightDirAndIntensityToWorldPoint( g_pLight[i].xyz , pWorldPosition, g_pLight[i].w, g_cLightDiffuse[i].w );

		// (N.L)
		float fNDotL = max( 0, dot( vWorldNormal, vLight.xyz ) );
	
		// diffuse color
		cDiffuseColor += g_cLightDiffuse[i].rgb * vLight.w * fNDotL * g_cMaterialDiffuse;
	}
	// send alpha down with color
	vsOut.cDiffuse = float4( cDiffuseColor , g_cMaterialDiffuse.a );
	
	// if cloaking is enabled, this object needs to be distorted so that its depth value is 
    // written correctly into the depth buffer; the cloaking effect expects the depth buffer 
    // to be established before it begins its processing - jmg
    if (g_bEnableCloakDistortion)
    {
    	// distort the model in screen space - jmg
		vsOut.vPosition.xy += ComputeCloakingDistortion(vsOut.vPosition, g_fDistortionWaveAmplitude, g_CloakTime);
	}
	
	vsOut.fPhazeOffset = MakePhaseSeed( vTextureUV, vPosition, g_fPulseSpeed1, g_fPulseScale1 );
	
	return vsOut;
}

///////////////////////////////////////////////////////////////////////////////
// Pixel Shader
///////////////////////////////////////////////////////////////////////////////
// Modulates the diffuse color of the vertex with the diffuse map
float4 PS_DiffuseModulate( VS_OUTPUT_LIT psIn ): COLOR0
{
	float fire2Tiling = g_fDecalTiling * 0.5f;
 	float4 finalColor = tex2D( g_sampDiffuse, psIn.vTextureUV * float2(g_fDecalTiling, g_fDecalTiling) )* psIn.cDiffuse;
 	float4 fireColor1 = tex2D( g_sampSpecular, psIn.vTextureUV * float2(g_fDecalTiling, g_fDecalTiling)) * psIn.fPhazeOffset.x;	
	float4 fireColor2 = tex2D( g_sampDetail, psIn.vTextureUV * float2(fire2Tiling, fire2Tiling) ) * psIn.fPhazeOffset.x * 2;
	float4 sparksColor = tex2D( g_sampNormal, psIn.vTextureUV * float2(g_fDecalTiling, g_fDecalTiling) );
 	
 	// force the final color to transition to black as the object becomes more transparent - jmg
 	finalColor.rgb -= 1.0f - g_fOpacity;
	finalColor.rgb = saturate(finalColor.rgb);
	finalColor.a = 1.0f;

	// add in the color of the fire and sparks	
	finalColor.rgb += fireColor1.rgb * g_fOpacity;
	finalColor.rgb += fireColor2.rgb * 2 * g_fOpacity;
	finalColor.rgb += sparksColor * g_fOpacity;
 	
	// just light texture and output material alpha too with texture alpha
	return finalColor;
}

float4 PS_Sparks( VS_OUTPUT psIn ): COLOR0
{
	float4 finalColor = tex2D( g_sampNormal, psIn.vTextureUV * float2(g_fDecalTiling, g_fDecalTiling) );
	
	// force the final color to transition to black as the object becomes more transparent - jmg
	finalColor.rgb -= 1.0f - g_fOpacity;
	finalColor.rgb = saturate(finalColor.rgb);

	// only the texture unlit
	// we use the normal sampler to store the spark texture
	return finalColor;
}

float4 PS_Fire1( VS_OUTPUT psIn ): COLOR0
{
	float4 finalColor = tex2D( g_sampSpecular, psIn.vTextureUV * float2(g_fDecalTiling, g_fDecalTiling)) * psIn.fPhazeOffset.x;
	
	// force the final color to transition to black as the object becomes more transparent - jmg
	finalColor.a *= g_fOpacity;
	finalColor.rgb -= 1.0f - g_fOpacity;
	finalColor.rgb = saturate(finalColor.rgb);	
	
	return finalColor;
}
float4 PS_Fire2( VS_OUTPUT psIn ): COLOR0
{
	float4 finalColor = tex2D( g_sampDetail, psIn.vTextureUV * float2(g_fDecalTiling, g_fDecalTiling) ) * psIn.fPhazeOffset.x;
	
	// force the final color to transition to black as the object becomes more transparent - jmg
	finalColor.a *= g_fOpacity;
	finalColor.rgb -= 1.0f - g_fOpacity;
	finalColor.rgb = saturate(finalColor.rgb);	
	
	// only the texture unlit
	// we use the normal sampler to store the spark texture
	return finalColor;
}
///////////////////////////////////////////////////////////////////////////////
// default Effect
///////////////////////////////////////////////////////////////////////////////

// renders the object with just a diffuse texture
technique BaseOnly
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1";

// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = 1;
// the number of lights the effect can do
// this effect ignores lights so it can render any number ...
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
	pass BasePass
	{
		AlphaBlendEnable = true;
		ZFunc = LESSEQUAL;
		ZEnable = true;
		ZWriteEnable = true;
		SrcBlend = SRCALPHA;
		DestBlend = INVSRCALPHA;
		VertexShader = compile vs_3_0 VS_TransformLitTex1( );
		PixelShader = compile ps_3_0 PS_DiffuseModulate( );
	}
}
technique BaseSpark
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1";

// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = 1;
// the number of lights the effect can do
// this effect ignores lights so it can render any number ...
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
	pass BasePass
	{
		AlphaBlendEnable = true;
		ZFunc = LESSEQUAL;
		ZEnable = true;
		ZWriteEnable = true;
		SrcBlend = SRCALPHA;
		DestBlend = INVSRCALPHA;
		VertexShader = compile vs_3_0 VS_TransformLitTex1( );
		PixelShader = compile ps_3_0 PS_DiffuseModulate( );
	}
	pass SparkPass
	{
		AlphaBlendEnable = true;
		ZFunc = LESSEQUAL;
		ZEnable = true;
		ZWriteEnable = false;
		SrcBlend = SRCALPHA;
		DestBlend = one;
		VertexShader = compile vs_3_0 VS_Fire1( );
		PixelShader = compile ps_3_0 PS_Sparks( );
	}
}

technique BaseFire1
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1";

// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = 1;
// the number of lights the effect can do
// this effect ignores lights so it can render any number ...
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
	pass BasePass
	{
		AlphaBlendEnable = true;
		ZFunc = LESSEQUAL;
		ZEnable = true;
		ZWriteEnable = true;
		SrcBlend = SRCALPHA;
		DestBlend = INVSRCALPHA;
		VertexShader = compile vs_3_0 VS_TransformLitTex1( );
		PixelShader = compile ps_3_0 PS_DiffuseModulate( );
	}
	pass Fire1
	{
		AlphaBlendEnable = true;
		ZFunc = LESSEQUAL;
		ZEnable = true;
		ZWriteEnable = false;
		SrcBlend = one;
		DestBlend = one;
		VertexShader = compile vs_3_0 VS_Fire1( );
		PixelShader = compile ps_3_0 PS_Fire1( );
	}
}

technique BaseFire2
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1";

// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = 1;
// the number of lights the effect can do
// this effect ignores lights so it can render any number ...
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
	pass BasePass
	{
		AlphaBlendEnable = true;
		ZFunc = LESSEQUAL;
		ZEnable = true;
		ZWriteEnable = true;
		SrcBlend = SRCALPHA;
		DestBlend = INVSRCALPHA;
		VertexShader = compile vs_3_0 VS_TransformLitTex1( );
		PixelShader = compile ps_3_0 PS_DiffuseModulate( );
	}
}

technique BaseSparkFire2
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1";

// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = 1;
// the number of lights the effect can do
// this effect ignores lights so it can render any number ...
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
	pass BasePass
	{
		AlphaBlendEnable = true;
		ZFunc = LESSEQUAL;
		ZEnable = true;
		ZWriteEnable = true;
		SrcBlend = SRCALPHA;
		DestBlend = INVSRCALPHA;
		VertexShader = compile vs_3_0 VS_TransformLitTex1( );
		PixelShader = compile ps_3_0 PS_DiffuseModulate( );
	}	
}




///////////////////////////////////////////////////////////////////////////////
// this value must exactly match the enum value of
// The output of the Vertex Shader
struct VS_OUTPUT_FallBack
{
  // The transformed position
  float4 vPosition : POSITION;
  // The diffuse color 
  float4 cDiffuse : COLOR0;
 // The UV coordinates of the texture.
  float2 tcDiffuse : TEXCOORD0;
};

///////////////////////////////////////////////////////////////////////////////

// simplest vertex shader just transforms and ignores normals
VS_OUTPUT_FallBack VS_FallBack(
  const float3 vPosition : POSITION,
  const float3 vNormal : NORMAL,
  const float2 tcDiffuse : TEXCOORD0
  )
{
	VS_OUTPUT_FallBack vsOut;
	vsOut.vPosition = ModelToScreenSpace( vPosition );
	// output constant diffuse color
	vsOut.cDiffuse = g_cMaterialDiffuse + g_cMaterialAmbient;

	// output texture coordinates
	vsOut.tcDiffuse = tcDiffuse;
	return vsOut;
}


float4 PS_FallBack( VS_OUTPUT_FallBack psIn ): COLOR0
{
	// light the texture
	return tex2D( g_sampDiffuse, psIn.tcDiffuse )*psIn.cDiffuse;
}

technique FallBack
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1";

// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = 0;
// the number of lights the effect can do
// this effect ignores lights so it can render any number ...
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
	pass BasePass
	{
		AlphaBlendEnable = true;
		ZFunc = LESSEQUAL;
		ZEnable = true;
		ZWriteEnable = true;
		SrcBlend = SRCALPHA;
		DestBlend = INVSRCALPHA;
		VertexShader = compile vs_3_0 VS_FallBack( );
		PixelShader = compile ps_3_0 PS_FallBack( );
	}	
}