///////////////////////////////////////////////////////////////////////////////
// Vertex Shader
///////////////////////////////////////////////////////////////////////////////
#define _USE_WVP
#define _USE_W
#define _USE_VInv
#define _USE_MATERIAL
#define _USE_WInvTrans
#define _USE_LIGHTS
#define _USE_EYEPOS
#define _USE_CLOAKING
#define _USE_UVOFFSET
#define _USE_DIFFUSEMAP
#define _USE_SHADOWS

#include "StandardParameters.fxh"
#include "effect_common.fxh"
#include "Shadows.fxh"


///////////////////////////////////////////////////////////////////////////////
// this value must exactly match the enum value of
// The output of the Vertex Shader
struct VS_OUTPUT
{
  // The transformed position
  float4 vPosition : POSITION;
  // The diffuse color 
  float4 cDiffuse : COLOR0;
  // specular
  float3 cSpecular : COLOR1;
 // The UV coordinates of the texture.
  float2 tcDiffuse : TEXCOORD0;
  float4 tcShadowMap : TEXCOORD1;
  float3 cAmbient : TEXCOORD2;
};

///////////////////////////////////////////////////////////////////////////////

// simplest vertex shader just transforms and ignores normals
VS_OUTPUT VS_TransformedTex(
  const float3 vPosition : POSITION,
  const float3 vNormal : NORMAL,
  const float2 tcDiffuse : TEXCOORD0,
  uniform bool bDoShadows,
  uniform int iLightCount )
{
	VS_OUTPUT vsOut;
	vsOut.vPosition = ModelToScreenSpace( vPosition );
	// get world position for diffuse lighting
	float4 pWorldPosition = ModelToWorldSpace( vPosition );
	if( bDoShadows && g_TextureSize )
	{	
		// generate shadow space pos
		vsOut.tcShadowMap = WorldSpaceToShadowSpace ( pWorldPosition );
		// make z linear
		vsOut.tcShadowMap.z/=DEPTH_SCALE;
		vsOut.tcShadowMap.w/=DEPTH_SCALE;
	}
	else
	{
		vsOut.tcShadowMap = 0;
	}
	// start at black then add the lights
	float3 cDiffuseColor = 0;
	float3 cSpecularColor = 0;
	
	vsOut.cAmbient = g_cMaterialAmbient;
	
	// bring normal to world
	float3 vWorldNormal = ModelToWorldSpaceRotate( vNormal );
	
	// for each light just compute color
	for( int i = 0; i <iLightCount; i++ ) 
	{
		// light intensity and direction from vert pos and light pos
		float4 vLight = ComputeLightDirAndIntensityToWorldPoint( g_pLight[i].xyz , pWorldPosition, g_pLight[i].w, g_cLightDiffuse[i].w );

		// (N.L)
		float fNDotL = max( 0, dot( vWorldNormal, vLight.xyz ) );
	
		// diffuse color
		cDiffuseColor += g_cLightDiffuse[i].rgb * vLight.w * fNDotL * g_cMaterialDiffuse;
		
		// (N.H) specular
		float3 H = ComputeHalfAngle( vLight.xyz , pWorldPosition );
		float fNDotH = max( 0, dot( vWorldNormal, H ) );
		
		// if diffuse does not light the spot at all specular should not
		if( fNDotL <= 0 )
		{
			fNDotH = 0;
		}
	
		// We use specMag^(shininess) for our fall-off.
		float fNDotHExp = pow( fNDotH, shininess );
		cSpecularColor += g_cLightDiffuse[i].rgb * g_cMaterialSpecular * vLight.w * fNDotHExp;
		
	}
	// send alpha down with color
	vsOut.cDiffuse = min( float4(1,1,1,1) , float4( cDiffuseColor , g_cMaterialDiffuse.a ) );
	
	// output specular color
	vsOut.cSpecular = min( float4(1,1,1,1) , cSpecularColor );
	
	// if cloaking is enabled, this object needs to be distorted so that its depth value is 
    // written correctly into the depth buffer; the cloaking effect expects the depth buffer 
    // to be established before it begins its processing - jmg
    if (g_bEnableCloakDistortion)
    {
    	// distort the model in screen space - jmg
		vsOut.vPosition.xy += ComputeCloakingDistortion(vsOut.vPosition.xy, g_fDistortionWaveAmplitude, g_CloakTime);
	}
	
	// output texture coordinates
	vsOut.tcDiffuse = tcDiffuse+g_DiffuseUVOffset;
	return vsOut;
}


///////////////////////////////////////////////////////////////////////////////
// Pixel Shader
///////////////////////////////////////////////////////////////////////////////

// Modulates the diffuse color of the vertex with the diffuse map
float4 PS_Diffuse(VS_OUTPUT psIn,
				  uniform float4 cMaterialDiffuse,
				  uniform bool bDoShadows ) : COLOR0
{
	// light the texture and use the material's alpha value
	float4 result = tex2D( g_sampDiffuse, psIn.tcDiffuse );
	if( g_bUseTextureAlpha )
	{
		// mul with tex alpha
		result.a *= psIn.cDiffuse.a;
	}
	else
	{
		// use global alpha
		result.a = psIn.cDiffuse.a;
	}
	if( bDoShadows && g_TextureSize )
	{
		// do 4 sample PCF
		float shadow = ComputeShadow4PCF( psIn.tcShadowMap );
    	psIn.cDiffuse.rgb *= shadow;
    	psIn.cSpecular.rgb *= shadow;
	}
	result.rgb = result.rgb*(psIn.cDiffuse.rgb + psIn.cAmbient.rgb) + psIn.cSpecular;	
	
	return result;
}

///////////////////////////////////////////////////////////////////////////////

// renders the object with just a diffuse texture
technique RenderDiffuse
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_DEFAULT;
// the number of lights the effect can do
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
	// render the diffuse texture
	pass DiffusePass1
	{
		VertexShader = compile vs_3_0 VS_TransformedTex( true, 8 );
		PixelShader = compile ps_3_0 PS_Diffuse(g_cMaterialDiffuse, true);
	}
}

// renders the object with just a diffuse texture
technique RenderDiffuse_noshadow
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_LOW;
// the number of lights the effect can do
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
	// render the diffuse texture
	pass DiffusePass1
	{
		VertexShader = compile vs_3_0 VS_TransformedTex( false, 1 );
		PixelShader = compile ps_3_0 PS_Diffuse(g_cMaterialDiffuse, false);
	}
}
///////////////////////////////////////////////////////////////////////////////

// Default lighting.
#include "FixedPipelineDiffuse.fxh"

///////////////////////////////////////////////////////////////////////////////

