#define _USE_WVP
#define _USE_W
#define _USE_WTrans
#define _USE_DIFFUSEMAP
#define _USE_NORMALMAP
#define _USE_DETAILMAP
#define _USE_MATERIAL
#define _USE_LIGHTS
#define _USE_ILLUMINATIONMAP
#define _USE_EYEPOS
#define _USE_CLOAKING

#include "diffusebump.fxh"

///////////////////////////////////////////////////////////////////////////////

struct diffuseDetailVertexOutput 
{
    float4 pPosition		: POSITION;
    float2 tcDiffuse		: TEXCOORD0;
    float2 tcNormal			: TEXCOORD1;
    float2 tcDetail			: TEXCOORD2;
    float4 vLight			: TEXCOORD3;
    float4 vHalfAngle		: TEXCOORD4;
};


///////////////////////////////////////////////////////////////////////////////
diffuseDetailVertexOutput VS_DiffuseNormalDetail( vertexInput_ST IN ,
													uniform float4 vLightPos,
													uniform float4 vLightColor )
{
    diffuseDetailVertexOutput OUT;

	// Transform position to clip space and output it
    OUT.pPosition = ModelToScreenSpace( IN.pPosition );
    
    // Output texture coordinates.
	OUT.tcDiffuse = IN.tcDiffuse;
	OUT.tcNormal = IN.tcDiffuse;
	
	// compute and output the detail coordinates
	OUT.tcDetail = DiffuseToDetailCoord( IN.tcDiffuse );
	
	// get world position for diffuse and specular lighting
	float4 pWorldPosition = ModelToWorldSpace( IN.pPosition );
	
	// light intensity and direction from pos and light pos
	float4 vLight = ComputeLightDirAndIntensityToWorldPoint( vLightPos.xyz , pWorldPosition.xyz, vLightPos.w, vLightColor.w );
	
	// Put the unpacked normal space light vector in diffuse 0
	// put intensity in w
	float3 vNormalSpaceLight = ToNormalSpace( WorldToModelSpaceRotate( vLight.xyz ) , IN.vNormal, IN.vTangent, IN.vBinormal );
	// we really should normalize but we need to fix the planet tangent and binormal on the cube seams before we do
	OUT.vLight = float4( vNormalSpaceLight,  vLight.w );
	
	// Put the half-angle vector in diffuse 1 for specular
	float3 vNormalSpaceHalfAngle = ToNormalSpace( WorldToModelSpaceRotate( ComputeHalfAngle( vLight.xyz , pWorldPosition ) ), IN.vNormal, IN.vTangent, IN.vBinormal );
	OUT.vHalfAngle =  float4( vNormalSpaceHalfAngle , 1 );
	
	return OUT;
}



float4 PS_DiffuseNormalDetailAmb( diffuseDetailVertexOutput IN,
											uniform float3 cLightColor,
											uniform float4 cMaterialDiffuse, // we want alpha mat value to carry through
											uniform float3 cMaterialAmbient ) : COLOR
{
	// fetch base texture
	float4 cDiffTex = tex2D( g_sampDiffuse, IN.tcDiffuse );
	// fetch detail texture
	float4 cDetTex = tex2D( g_sampDetail, IN.tcDetail );
	
	// The effective diffuse texture color
	float4 cDiffMat = cDiffTex*(cDetTex*g_cDetailScalar+g_cDetailOffset);
	
	// fetch normal texture
	float4 vNormal = tex2D( g_sampNormal, IN.tcNormal );
	// Unpack our 2 vectors.
	vNormal = Unpack( vNormal );
		
	// fetch second texture lookup
	float fNDotL = saturate( dot( vNormal.xyz, IN.vLight.xyz ) );
	float2 uv = float2(	fNDotL, dot( vNormal.xyz, IN.vHalfAngle.xyz ) );
	
	// The xyz (rgb) term is the incident light magnitude,
	// and the w (a) term is the square root of the percent
	// reflected towards the viewer.
	float4 cSpecLight =
		tex2D( g_sampIllumination, uv );

	// specular color
	float fSpecPow = cSpecLight.a*cSpecLight.a;
	float3 cSpecular = fSpecPow*cLightColor*g_cMaterialSpecular;
	
	// diffuse color
	float3 cDiffuse = cMaterialDiffuse*cLightColor*fNDotL*IN.vLight.w;
	
	// modulate lighting and modulate material and texture alpha
	return float4( saturate(cDiffuse+cMaterialAmbient) * cDiffMat + cSpecular,  cMaterialDiffuse.a );
}

// renders the object with just a diffuse texture
technique RenderDiffuseNormalDetail
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1ST";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_DEFAULT;
// the number of lights the effect can do
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
	// render the diffuse texture
	pass BasePass
	<
    	// tell the mad3D engine that this is the base pass 
    	// to do once
    	bool bBasePass = true;
	>
	{
		VertexShader = compile vs_3_0 VS_DiffuseNormalDetail( (g_pBasePassLightPosition) , (g_cBasePassLightColor) );
		PixelShader = compile ps_3_0 PS_DiffuseNormalDetailAmb( (g_cBasePassLightColor), (g_cMaterialDiffuse), (g_cMaterialAmbient) );
	}
	pass PerLightPass
	<
	// tell the mad3D engine pass to call once for every light while changing the value 
	// of the g_vLightColor & g_vLightPosition parameters
	bool bPerLightPass = true;
	>
	{
    	AlphaBlendEnable = true;
		DestBlend = one;
		VertexShader = compile vs_3_0 VS_DiffuseNormalDetail( (g_pPerLightPassLightPosition) , (g_cPerLightPassLightColor) );
		// no ambient on subsequent passes
		PixelShader = compile ps_3_0 PS_DiffuseNormalDetailAmb( (g_cPerLightPassLightColor), (g_cMaterialDiffuse), float3( 0,0,0 ) );
	}
}


// Bump mapping with no detail map for shader_1_1
technique DiffuseBump
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1ST";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_LOW;
// the number of lights the effect can do
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
   pass ShadowPass
    <
    // this annotation tells the Mad3D_Renderer that this is the base pass to use only once
    bool	bShadowPass = true;
    >
    {
		VertexShader = compile vs_3_0 VS_BumpDiffuse((g_pBasePassLightPosition) , (g_cBasePassLightColor), false);
		PixelShader  = compile ps_3_0 PS_BumpDiffuse((g_cBasePassLightColor), (g_cMaterialDiffuse), (g_cMaterialAmbient), false);
    }
    pass BasePass
	<
    	// tell the mad3D engine that this is the base pass 
    	// to do once
    	bool bBasePass = true;
	>
    {
		VertexShader = compile vs_3_0 VS_BumpDiffuse( (g_pBasePassLightPosition) , (g_cBasePassLightColor), false );
		PixelShader  = compile ps_3_0 PS_BumpDiffuse( (g_cBasePassLightColor), (g_cMaterialDiffuse), (g_cMaterialAmbient), false );
    }
	pass PerLightPass
	<
	// tell the mad3D engine pass to call once for every light while changing the value 
	// of the g_vLightColor & g_vLightPosition parameters
	bool bPerLightPass = true;
	>
    {
    	AlphaBlendEnable = true;
		DestBlend = one;
		VertexShader = compile vs_3_0 VS_BumpDiffuse( (g_pPerLightPassLightPosition) , (g_cPerLightPassLightColor), false );
		PixelShader  = compile ps_3_0 PS_BumpDiffuse( (g_cPerLightPassLightColor), (g_cMaterialDiffuse), float3( 0,0,0 ), false );
    }
}



///////////////////////////////////////////////////////////////////////////////

// Default lighting.
#include "FixedPipelineDiffuse.fxh"

///////////////////////////////////////////////////////////////////////////////

