
#define _USE_VInv
#define _USE_SHADOWS
#define _USE_WVP
#define _USE_CLOAKING
#define _USE_W
#define _USE_WTrans
#define _USE_SPECMAP
#define _USE_DIFFUSEMAP
#define _USE_NORMALMAP
#define _USE_CLIPPLANE
#define _USE_MATERIAL
#define _USE_LIGHTS


// the implementation is in this file
#include "diffusebump.fxh"

// The current file just makes it easy for the game to find and validate specific implementations
// this very files should contain all variants and fallback for an object with
// a diffuse map, a normal map and a specular map.

struct diffuseNormalSpecularVertexOutput {
    float4 pPosition		: POSITION;
    float2 tcDiffuse		: TEXCOORD0;
    float2 tcNormal			: TEXCOORD1;
    float2 tcSpecular		: TEXCOORD2;
    float4 tcShadowMap		: TEXCOORD3;
    float4 tcWorldPosition	: TEXCOORD4;
    float4 vLight			: COLOR0;
    float4 vHalfAngle		: COLOR1;
};


///////////////////////////////////////////////////////////////////////////////



diffuseNormalSpecularVertexOutput VS_DiffuseNormalSpecular( vertexInput_ST IN,
															uniform float4 vLightPos,
															uniform float4 vLightColor,
															uniform bool bDoShadows )
{
     diffuseNormalSpecularVertexOutput OUT;
   
	// Transform position to clip space and output it
    OUT.pPosition = ModelToScreenSpace( IN.pPosition );
    
    // if cloaking is enabled, this object needs to be distorted so that its depth value is 
    // written correctly into the depth buffer; the cloaking effect expects the depth buffer 
    // to be established before it begins its processing - jmg
    if (g_bEnableCloakDistortion)
    {
    	// distort the model in screen space - jmg
		OUT.pPosition.xy += ComputeCloakingDistortion(OUT.pPosition.xy, g_fDistortionWaveAmplitude, g_CloakTime);
	}
   
    // Output texture coordinates.
	OUT.tcDiffuse = IN.tcDiffuse;
	OUT.tcNormal = IN.tcDiffuse;
	OUT.tcSpecular = IN.tcDiffuse;
	
	// get world position for diffuse and specular lighting
	float4 pWorldPosition = ModelToWorldSpace( IN.pPosition );
	OUT.tcWorldPosition = pWorldPosition;
	
	if( bDoShadows )
	{	
		// generate shadow space pos
		OUT.tcShadowMap = WorldSpaceToShadowSpace ( pWorldPosition );
		// make z linear
		OUT.tcShadowMap.z/=DEPTH_SCALE;
		OUT.tcShadowMap.w/=DEPTH_SCALE;
	}
	else
	{
		OUT.tcShadowMap = 0;
	}
	
	// light intensity and direction from pos and light pos
	float4 vLight = ComputeLightDirAndIntensityToWorldPoint( vLightPos.xyz , pWorldPosition.xyz, vLightPos.w, vLightColor.w );
	
	// Put the unpacked normal space light vector in diffuse 0
	// put intensity in w
	float3 vNormalSpaceLight = ToNormalSpace( WorldToModelSpaceRotate(vLight.xyz) , IN.vNormal, IN.vTangent, IN.vBinormal );
	OUT.vLight =  float4( vNormalSpaceLight,  vLight.w );

	// Put the half-angle vector in diffuse 1
	float3 vNormalSpaceHalfAngle = ToNormalSpace( WorldToModelSpaceRotate( ComputeHalfAngle( vLight.xyz , pWorldPosition ) ), IN.vNormal, IN.vTangent, IN.vBinormal );
	OUT.vHalfAngle = float4( vNormalSpaceHalfAngle , 1 );

	return OUT;
}


///////////////////////////////////////////////////////////////////////////////
float4 PS_DiffuseNormalSpecular( diffuseNormalSpecularVertexOutput IN,
											uniform float3 cLightColor,
											uniform float4 cMaterialDiffuse, // we want alpha mat value to carry through
											uniform float3 cMaterialAmbient,
											uniform bool bDoShadows ): COLOR
{
	// fetch base texture
	float4 cDiffTex =
		tex2D( g_sampDiffuse, IN.tcDiffuse );
	// fetch normal texture
	float4 vNormal =
		tex2D( g_sampNormal, IN.tcNormal );
	// fetch specular texture
	float4 cSpecTex =
		tex2D( g_sampSpecular, IN.tcSpecular );
		
	// Unpack our vector.
	vNormal.xyz = Unpack( vNormal.xyz );
	float4 vLight = IN.vLight;
	float3 vHalfAngle =  IN.vHalfAngle.xyz;

	// N.L
	float fNDotL = vLight.w*saturate( dot( vNormal.xyz, vLight.xyz ) );
	// N.H
	float fNDotH = vLight.w* saturate( dot( vNormal.xyz, vHalfAngle.xyz ) );
	if( fNDotL <= 0 )
	{
		fNDotH = 0;
	}
	else
	{
		if( bDoShadows )
		{
			// do 4 sample PCF
			#ifdef _XBOX
			float shadow = ShadowSample16( IN.tcShadowMap );
			#else
			float shadow = ComputeShadow4PCF( IN.tcShadowMap );
			#endif
	    	fNDotL *= shadow;
	    	fNDotH *= shadow;
	    }
	}
     
    // if the pixel falls behind the specified clipping plane, kill it
    clip(dot(IN.tcWorldPosition - g_vClipPlanePosition, g_vClipPlaneNormal));

	// We use specMag^(shininess) for our fall-off.
	float fNDotHExp = pow( fNDotH, shininess );
	
	// Compute the specular color
	float3 cSpecular = g_cMaterialSpecular.xyz * cSpecTex.xyz * cLightColor * fNDotHExp;
	
	// Compute the diffuse color, witht the per pass light color parameter
	float3 cDiffuse = cMaterialDiffuse.rgb * cLightColor * fNDotL;
	
	// Compute the ambient color, we do not use light ambient so it ends up 
	// being only material ambient, default black.
	float3 cAmbient = cMaterialAmbient.xyz;
		
	// combine lighting with diffuse and detail texture
	return float4( saturate(cDiffuse+cAmbient)*cDiffTex.rgb + cSpecular, cMaterialDiffuse.a );
}


///////////////////////////////////////////////////////////////////////////////

technique BumpSpecularFull
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1ST";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_DEFAULT;
// the number of lights the effect can do
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
   pass ShadowPass
    <
    // this annotation tells the Mad3D_Renderer that this is the base pass to use only once
    bool	bShadowPass = true;
    >
    {
		VertexShader = compile vs_3_0 VS_DiffuseNormalSpecular((g_pBasePassLightPosition) , (g_cBasePassLightColor), true);
		PixelShader  = compile ps_3_0 PS_DiffuseNormalSpecular((g_cBasePassLightColor), (g_cMaterialDiffuse), (g_cMaterialAmbient), true);
    }
    pass BasePass
    <
    // this annotation tells the Mad3D_Renderer that this is the base pass to use only once
    bool	bBasePass = true;
    >
    {
		VertexShader = compile vs_3_0 VS_DiffuseNormalSpecular((g_pBasePassLightPosition) , (g_cBasePassLightColor), false);
		PixelShader  = compile ps_3_0 PS_DiffuseNormalSpecular((g_cBasePassLightColor), (g_cMaterialDiffuse), (g_cMaterialAmbient), false);
    }
    pass PerLightPass
    <
    // this annotation tells the Mad3D_Renderer that this pass can be called 
    // once per additionnal light after the base pass
	bool bPerLightPass = true;    
    >
    {
    	AlphaBlendEnable = true;
		DestBlend = ONE;
		VertexShader = compile vs_3_0 VS_DiffuseNormalSpecular((g_pPerLightPassLightPosition) , (g_cPerLightPassLightColor), false);
		PixelShader  = compile ps_3_0 PS_DiffuseNormalSpecular((g_cPerLightPassLightColor), (g_cMaterialDiffuse), float3( 0,0,0 ), false);
    }
}

// Bump mapping with no detail map for shader_1_1
technique DiffuseBump
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1ST";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_LOW;
// the number of lights the effect can do
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
   pass ShadowPass
    <
    // this annotation tells the Mad3D_Renderer that this is the base pass to use only once
    bool	bShadowPass = true;
    >
    {
		VertexShader = compile vs_3_0 VS_BumpDiffuse((g_pBasePassLightPosition) , (g_cBasePassLightColor), false);
		PixelShader  = compile ps_3_0 PS_BumpDiffuse((g_cBasePassLightColor), (g_cMaterialDiffuse), (g_cMaterialAmbient), false);
    }
    pass BasePass
	<
    	// tell the mad3D engine that this is the base pass 
    	// to do once
    	bool bBasePass = true;
	>
    {
		VertexShader = compile vs_3_0 VS_BumpDiffuse( (g_pBasePassLightPosition) , (g_cBasePassLightColor), false );
		PixelShader  = compile ps_3_0 PS_BumpDiffuse( (g_cBasePassLightColor), (g_cMaterialDiffuse), (g_cMaterialAmbient), false );
    }
	pass PerLightPass
	<
	// tell the mad3D engine pass to call once for every light while changing the value 
	// of the g_vLightColor & g_vLightPosition parameters
	bool bPerLightPass = true;
	>
    {
    	AlphaBlendEnable = true;
		DestBlend = one;
		VertexShader = compile vs_3_0 VS_BumpDiffuse( (g_pPerLightPassLightPosition) , (g_cPerLightPassLightColor), false );
		PixelShader  = compile ps_3_0 PS_BumpDiffuse( (g_cPerLightPassLightColor), (g_cMaterialDiffuse), float3( 0,0,0 ), false );
    }
}


///////////////////////////////////////////////////////////////////////////////

// Default lighting.
#include "FixedPipelineDiffuse.fxh"

///////////////////////////////////////////////////////////////////////////////
