

#define _USE_WVP
#define _USE_W
#define _USE_WTrans
#define _USE_SPECMAP
#define _USE_DIFFUSEMAP
#define _USE_NORMALMAP
#define _USE_DETAILMAP
#define _USE_MATERIAL
#define _USE_LIGHTS
#define _USE_CLOAKING

// the implementation is in this file
#include "diffusebump.fxh"

//------------------------------------
struct diffuseNormalSpecularVertexOutput 
{
    float4 pPosition	: POSITION;
    float2 tcDiffuse	: TEXCOORD0;
    float2 tcNormal		: TEXCOORD1;
    float2 tcSpecular	: TEXCOORD2;
    float2 tcDetail		: TEXCOORD3;
    float4 vLight		: TEXCOORD4;
    float4 vHalfAngle	: TEXCOORD5;
};


diffuseNormalSpecularVertexOutput VS_DiffuseNormalSpecularDetail( vertexInput_ST IN ,
																	uniform float4 vLightPos,
																	uniform float4 vLightColor ) 
{
    diffuseNormalSpecularVertexOutput OUT;

	// Transform position to clip space and output it
    OUT.pPosition = ModelToScreenSpace( IN.pPosition );
    
    // Output texture coordinates.
	OUT.tcDiffuse = IN.tcDiffuse;
	OUT.tcNormal = IN.tcDiffuse;
	OUT.tcSpecular = IN.tcDiffuse;
	
	// compute and output the detail coordinates
	OUT.tcDetail = DiffuseToDetailCoord( IN.tcDiffuse );
	
	// get world position for diffuse and specular lighting
	float4 pWorldPosition = ModelToWorldSpace( IN.pPosition );
	
	// light intensity and direction from pos and light pos
	float4 vLight = ComputeLightDirAndIntensityToWorldPoint( vLightPos.xyz , pWorldPosition.xyz, vLightPos.w, vLightColor.w );
	
	// Put the unpacked normal space light vector in diffuse 0
	// put intensity in w
	float3 vNormalSpaceLight = ToNormalSpace( WorldToModelSpaceRotate(vLight.xyz) , IN.vNormal, IN.vTangent, IN.vBinormal );
	OUT.vLight = float4( vNormalSpaceLight,  vLight.w );

	// Put the half-angle vector in diffuse 1
	float3 vNormalSpaceHalfAngle = ToNormalSpace( WorldToModelSpaceRotate( ComputeHalfAngle( vLight.xyz , pWorldPosition ) ), IN.vNormal, IN.vTangent, IN.vBinormal );
	OUT.vHalfAngle = float4( vNormalSpaceHalfAngle , 1 );

	return OUT;
}

float4 PS_DiffuseNormalSpecularDetailAmb( diffuseNormalSpecularVertexOutput IN,
											uniform float3 cLightColor,
											uniform float4 cMaterialDiffuse, // we want alpha mat value to carry through
											uniform float3 cMaterialAmbient ): COLOR
{
	// fetch base texture
	float4 cDiffTex =
		tex2D( g_sampDiffuse, IN.tcDiffuse );
	// fetch normal texture
	float4 vNormal =
		tex2D( g_sampNormal, IN.tcNormal );
	// fetch specular texture
	float4 cSpecTex =
		tex2D( g_sampSpecular, IN.tcSpecular );
	// fetch detail texture9	
	float4 cDetailTex =
		tex2D( g_sampDetail, IN.tcDetail );

	// The effective diffuse texture color
	float4 cDiffMat = DetailColorScale( cDiffTex, cDetailTex );

	// Unpack our vector.
	vNormal = Unpack( vNormal );
	float4 vLight = IN.vLight;
	float4 vHalfAngle = IN.vHalfAngle;

	// Compute the magnitude of light hitting the surface.
	float fNDotL = saturate( dot( vNormal.xyz, vLight.xyz ) );
	// Compute the amount of specular light reflected towards the camera.
	float fNDotH = saturate( dot( vNormal.xyz, vHalfAngle.xyz ) );
	if( fNDotL <= 0 )
	{
		fNDotH = 0;
	}
	
	// We use specMag^(shininess) for our fall-off.
	float fNDotHExp = pow( fNDotH, shininess );
	
	// Compute the specular color
	// use white light color, actually looks better
	float3 cSpecular = cSpecTex.xyz * cLightColor * fNDotHExp * vLight.w;
	
	// Compute the diffuse color, witht the per pass light color parameter
	float3 cDiffuse = cMaterialDiffuse.rgb * cLightColor * fNDotL * vLight.w;
	
	// Compute the ambient color, we do not use light ambient so it end up 
	// being only material ambient, default black.
	float3 cAmbient = cMaterialAmbient.xyz;
		
	// combine lighting with diffuse and detail texture
	return float4( saturate(cDiffuse+cAmbient) * cDiffMat + cSpecular , cMaterialDiffuse.a );
}


//-----------------------------------
// Bump mapping with just diffuse information
technique DiffuseBumpSpecularDetail
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1ST";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_DEFAULT;
// the number of lights the effect can do
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
	// pass to use for first light
	pass BasePass
	<
    	// tell the mad3D engine that this is the base pass 
    	// to do once
    	bool bBasePass = true;
	>
    {
		VertexShader = compile vs_3_0 VS_DiffuseNormalSpecularDetail( (g_pBasePassLightPosition) , (g_cBasePassLightColor) );
		PixelShader  = compile ps_3_0 PS_DiffuseNormalSpecularDetailAmb( (g_cBasePassLightColor), (g_cMaterialDiffuse), (g_cMaterialAmbient) );
    }
    
    // pass to use for eah additionnal light
	pass PerLightPass
	<
	// tell the mad3D engine pass to call once for every light while changing the value 
	// of the g_vLightColor & g_vLightPosition parameters
	bool bPerLightPass = true;
	>
    {
    	AlphaBlendEnable = true;
		DestBlend = one;
    	VertexShader = compile vs_3_0 VS_DiffuseNormalSpecularDetail( (g_pPerLightPassLightPosition) , (g_cPerLightPassLightColor) );
		PixelShader  = compile ps_3_0 PS_DiffuseNormalSpecularDetailAmb( (g_cPerLightPassLightColor), (g_cMaterialDiffuse), float3( 0,0,0 ) );
    }
}


// Bump mapping with no detail map for shader_1_1
technique DiffuseBump
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1ST";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_LOW;
// the number of lights the effect can do
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
    pass BasePass
	<
    	// tell the mad3D engine that this is the base pass 
    	// to do once
    	bool bBasePass = true;
	>
    {
		VertexShader = compile vs_3_0 VS_BumpDiffuse( (g_pBasePassLightPosition) , (g_cBasePassLightColor), false );
		PixelShader  = compile ps_3_0 PS_BumpDiffuse( (g_cBasePassLightColor), (g_cMaterialDiffuse), (g_cMaterialAmbient), false );
    }
	pass PerLightPass
	<
	// tell the mad3D engine pass to call once for every light while changing the value 
	// of the g_vLightColor & g_vLightPosition parameters
	bool bPerLightPass = true;
	>
    {
    	AlphaBlendEnable = true;
		DestBlend = one;
		VertexShader = compile vs_3_0 VS_BumpDiffuse( (g_pPerLightPassLightPosition) , (g_cPerLightPassLightColor), false );
		PixelShader  = compile ps_3_0 PS_BumpDiffuse( (g_cPerLightPassLightColor), (g_cMaterialDiffuse), float3( 0,0,0 ), false );
    }
}


///////////////////////////////////////////////////////////////////////////////

// Default lighting.
#include "FixedPipelineDiffuse.fxh"

///////////////////////////////////////////////////////////////////////////////
