#define _USE_WVP
#define _USE_W
#define _USE_WInvTrans
#define _USE_EYEPOS
#define _USE_DIFFUSEMAP
#define _USE_DETAILMAP
#define _USE_MATERIAL
#define _USE_LIGHTS

#include "StandardParameters.fxh"
#include "Effect_common.fxh"

struct diffuseDetailVertexOutput 
{
    float4 pPosition	: POSITION;
    float2 tcDiffuse	: TEXCOORD0;
    float2 tcDetail		: TEXCOORD2;
    float4 cDiffuse		: COLOR0;
    float3 cSpecular	: COLOR1;
};


diffuseDetailVertexOutput VS_DiffuseDetail( vertexInput IN,
											uniform float3 cMaterialAmbient,
											uniform float4 pLightPos,
											uniform float4 cLightColor )
{
    diffuseDetailVertexOutput OUT;
    
 	// screen pos
    OUT.pPosition 	= ModelToScreenSpace( IN.pPosition );
    // basic UV
	OUT.tcDiffuse 	= IN.tcDiffuse;
	
	// scale diffuse texcoord for the detail map
	OUT.tcDetail 	= DiffuseToDetailCoord( IN.tcDiffuse );
	
	// get world position for diffuse and specular lighting
	float4 pWorldPosition = ModelToWorldSpace( IN.pPosition );
	
	// light intensity and direction from vert pos and light pos
	float4 vLight = ComputeLightDirAndIntensityToWorldPoint( pLightPos.xyz , pWorldPosition, pLightPos.w, cLightColor.w );
	
	// bring normal to world
	float3 vWorldNormal = ModelToWorldSpaceRotate( IN.vNormal );
	// compute the light incidence
	float fNDotL = max( 0, dot( vWorldNormal, vLight.xyz ) );
	
	// light material diffuse color*fNDotL*intensity + ambient
	OUT.cDiffuse.rgb = saturate(g_cMaterialDiffuse * cLightColor.rgb * fNDotL * vLight.w + cMaterialAmbient);
	
	// (N.H) specular
	float3 H = ComputeHalfAngle( vLight.xyz , pWorldPosition );
	float fNDotH = max( 0, dot( vWorldNormal, H ) );
	
	// if diffuse does not light the spot at all specular should not
	if( fNDotL <= 0 )
	{
		fNDotH = 0;
	}

	// We use specMag^(shininess) for our fall-off.
	float fNDotHExp = pow( fNDotH, shininess );
	OUT.cSpecular = cLightColor.rgb * g_cMaterialSpecular * vLight.w * fNDotHExp;

	
	// we pass material alpha down to the shader
	OUT.cDiffuse.a = g_cMaterialDiffuse.a;
	return OUT;
}

///////////////////////////////////////////////////////////////////////////////

// Diffuse: DiffTexColor * (DiffLightColor * (N . L) + AmbLightColor)
// The final color is diffuse.
float4 PS_DiffuseDetailScale( diffuseDetailVertexOutput IN ): COLOR
{
	// fetch base texture
	float4 cDiffTex = tex2D( g_sampDiffuse, IN.tcDiffuse );
	// fetch detail texture
	float4 cDetTex = tex2D( g_sampDetail, IN.tcDetail );
	
	// The effective diffuse texture color
	float4 cDiffDetail = DetailColorScale( cDiffTex, cDetTex );

	// light the final diff and detail color
	return float4( cDiffDetail.rgb * IN.cDiffuse.rgb + IN.cSpecular, IN.cDiffuse.a );
}

///////////////////////////////////////////////////////////////////////////////

// renders the object with just a diffuse texture
technique RenderDiffuseDetail
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_DEFAULT;
// the number of lights the effect can do
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
	// pass to use for first light
	pass BasePass
	<
    	// tell the mad3D engine that this is the base pass 
    	// to do once
    	bool bBasePass = true;
	>
	{
		VertexShader = compile vs_3_0 VS_DiffuseDetail( g_cMaterialAmbient,g_pBasePassLightPosition, g_cBasePassLightColor );
		PixelShader = compile ps_3_0 PS_DiffuseDetailScale();
	}
    // pass to use for each additionnal light
	pass PerLightPass
	<
	// tell the mad3D engine pass to call once for every light while changing the value 
	// of the g_vLightColor & g_vLightPosition parameters
	bool bPerLightPass = true;
	>
	{
		AlphaBlendEnable = true;
		DestBlend = one;
		VertexShader = compile vs_3_0 VS_DiffuseDetail( float3(0,0,0),g_pPerLightPassLightPosition, g_cPerLightPassLightColor );
		PixelShader = compile ps_3_0 PS_DiffuseDetailScale();
	}
}

///////////////////////////////////////////////////////////////////////////////

// Default lighting.
#include "FixedPipelineDiffuse.fxh"

///////////////////////////////////////////////////////////////////////////////
