#define _USE_WVP
#define _USE_W
#define _USE_WInvTrans
#define _USE_SPECMAP
#define _USE_DIFFUSEMAP
#define _USE_MATERIAL
#define _USE_LIGHTS

#include "StandardParameters.fxh"
#include "Effect_common.fxh"


struct diffuseSpecularVertexOutput 
{
    float4 pPosition		: POSITION;
    float2 tcDiffuse		: TEXCOORD0;
    float3 vHalfAngle		: TEXCOORD1;
    float2 tcSpecular		: TEXCOORD2;
    float4 vLight			: COLOR0;
    float3 vNormal			: COLOR1;
};


///////////////////////////////////////////////////////////////////////////////
// a specular with interpolater normal and light dir
// looks better then linear specular colr interpolation across vertices
diffuseSpecularVertexOutput VS_DiffuseSpecular( vertexInput IN,
											uniform float4 pLightPos,
											uniform float4 cLightColor )
{
    diffuseSpecularVertexOutput OUT;
    
	// Transform position to clip space and output it
    OUT.pPosition = ModelToScreenSpace( IN.pPosition );
    
    // Output texture coordinates.
	OUT.tcDiffuse = IN.tcDiffuse;
	OUT.tcSpecular = IN.tcDiffuse;
	
	// get world position for diffuse and specular lighting
	float4 pWorldPosition = ModelToWorldSpace( IN.pPosition );
	// light intensity and direction from vert pos and light pos
	float4 vLight = ComputeLightDirAndIntensityToWorldPoint( pLightPos.xyz , pWorldPosition, pLightPos.w, cLightColor.w );
	
	// bring normal to world
	float3 vWorldNormal = ModelToWorldSpaceRotate( IN.vNormal );
	
	// pack out world normal
	OUT.vNormal = Pack( vWorldNormal ); 
	
	// Put the packed light vector in diffuse 0
	OUT.vLight = Pack( vLight );
	
	// Put the half-angle vector in diffuse 1
	OUT.vHalfAngle = Pack( float4( ComputeHalfAngle( vLight.xyz , pWorldPosition ) , 1 ) );

	return OUT;
}


///////////////////////////////////////////////////////////////////////////////
float4 PS_DiffuseSpecular( diffuseSpecularVertexOutput IN,
											uniform float3 cLightColor,
											uniform float4 cMaterialDiffuse, // we want alpha mat value to carry through
											uniform float3 cMaterialAmbient ): COLOR
{
	// fetch base texture
	float4 cDiffTex = tex2D( g_sampDiffuse, IN.tcDiffuse );
	// fetch specular texture
	float4 cSpecTex = tex2D( g_sampSpecular, IN.tcSpecular );

	// Unpack our vector TODO? and normalize, texture register interpolation denormalizes the vector
	// we can remove this normalize but the highlight shows artifacts
	float3 vNormal = normalize( Unpack( IN.vNormal ) );
	
	// Compute the magnitude of light hitting the surface.
	float fNDotL = saturate( dot( vNormal.xyz, Unpack( IN.vLight.xyz ) ) );
	// Compute the amount of specular light reflected towards the camera.
	float fNDotH = saturate( dot( vNormal.xyz, Unpack( IN.vHalfAngle ) ) );
	
	// if backfaced to light no specular
	if( fNDotL <= 0 )
	{
		fNDotH = 0;
	}
	
	// We use specMag^(shininess) for our fall-off.
	float fNDotHExp = pow( fNDotH, shininess );
	
	// Compute the specular color
	// using white as the specular color seems to work well most of the time
	// consider that instead of the light color
	float3 cSpecular = cSpecTex.rgb * cLightColor * fNDotHExp * IN.vLight.w;
	
	// Compute the diffuse color
	float3 cDiffuse = cMaterialDiffuse.rgb * cLightColor * fNDotL * IN.vLight.w;
	
	// modulate texture and lighting
	return float4( saturate( cDiffuse + cMaterialAmbient) * cDiffTex + cSpecular, cMaterialDiffuse.a );
}


///////////////////////////////////////////////////////////////////////////////
struct diffuseVertexSpecularVertexOutput 
{
    float4 pPosition		: POSITION;
    float2 tcDiffuse		: TEXCOORD0;
    float2 tcSpecular		: TEXCOORD1;
    float4 cSpecular		: COLOR0;
    float4 cDiffuse			: COLOR1;
};

// diffuse specular using a texture for a look up
diffuseVertexSpecularVertexOutput VS_DiffuseVertexSpecular( vertexInput IN,
											uniform float4 pLightPos,
											uniform float4 cLightColor )
{
    diffuseVertexSpecularVertexOutput OUT;
    
	// Transform position to clip space and output it
    OUT.pPosition = ModelToScreenSpace( IN.pPosition );
    
    // Output texture coordinates.
	OUT.tcDiffuse = IN.tcDiffuse;
	OUT.tcSpecular = IN.tcDiffuse;
	
	// get world position for diffuse and specular lighting
	float4 pWorldPosition = ModelToWorldSpace( IN.pPosition );
	// light intensity and direction from vert pos and light pos
	float4 vLight = ComputeLightDirAndIntensityToWorldPoint( pLightPos.xyz , pWorldPosition, pLightPos.w, cLightColor.w );

	// bring normal to world
	float3 vWorldNormal = ModelToWorldSpaceRotate( IN.vNormal );
	
	// (N.L)
	float fNDotL = max( 0, dot( vWorldNormal.xyz, vLight.xyz ) );
	// (N.H)
	float fNDotH = max( 0, dot( vWorldNormal.xyz, ComputeHalfAngle( vLight.xyz , pWorldPosition ) ) );
	if( fNDotL <= 0 )
	{
		fNDotH = 0;
	}
	
	// diffuse color
	OUT.cDiffuse =  cLightColor * vLight.w * fNDotL;
	
	float fNDotHExp = pow( fNDotH, shininess );

	// specular color
	OUT.cSpecular = g_cMaterialSpecular*fNDotHExp*vLight.w;

	return OUT;
}


///////////////////////////////////////////////////////////////////////////////
// this shader sucks, do a better one or
float4 PS_DiffuseVertexSpecular( diffuseVertexSpecularVertexOutput IN,
											uniform float3 cLightColor,
											uniform float4 cMaterialDiffuse, // we want alpha mat value to carry through
											uniform float3 cMaterialAmbient ): COLOR
{
	// fetch base texture
	float4 cDiffTex = tex2D( g_sampDiffuse, IN.tcDiffuse );
		// fetch specular texture
	float4 cSpecTex = tex2D( g_sampSpecular, IN.tcSpecular );
	
	// Compute the lighting component.
	return float4( saturate( IN.cDiffuse.xyz + cMaterialAmbient) * cDiffTex + IN.cSpecular.xyz*cSpecTex, cDiffTex.a*cMaterialDiffuse.a );
}

///////////////////////////////////////////////////////////////////////////////

// put the 2.0 implementation first, we will grab the first technique 
// that is supported thus giving us the better looking one

// Dot3 Bump mapping with diffuse & specular textures and a normal
// map.  Computes the specular reflection in the pixel shader.
technique SpecularFull
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_DEFAULT;
// the number of lights the effect can do
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
    pass BasePass
	<
    	// tell the mad3D engine that this is the base pass 
    	// to do once
    	bool bBasePass = true;
	>
    {
		VertexShader = compile vs_3_0 VS_DiffuseSpecular( (g_pBasePassLightPosition) , (g_cBasePassLightColor) );
		PixelShader  = compile ps_3_0 PS_DiffuseSpecular( (g_cBasePassLightColor), (g_cMaterialDiffuse), (g_cMaterialAmbient) );
    }
    pass PerLightPass
	<
	// tell the mad3D engine pass to call once for every light while changing the value 
	// of the g_vLightColor & g_vLightPosition parameters
	bool bPerLightPass = true;
	>
	{
    	AlphaBlendEnable = true;
		DestBlend = one;
		VertexShader = compile vs_3_0 VS_DiffuseSpecular((g_pPerLightPassLightPosition) , (g_cPerLightPassLightColor));
		PixelShader  = compile ps_3_0 PS_DiffuseSpecular((g_cPerLightPassLightColor), (g_cMaterialDiffuse), float3( 0,0,0 ));
    }
}


///////////////////////////////////////////////////////////////////////////////
	
// simple vertex specular
technique VertexSpecular
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_LOW;
// the number of lights the effect can do
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
    pass BasePass
	<
    	// tell the mad3D engine that this is the base pass 
    	// to do once
    	bool bBasePass = true;
	>
	{
    	AlphaBlendEnable = false;
		ZFunc = LESSEQUAL;
		ZEnable = true;
		VertexShader = compile vs_3_0 VS_DiffuseVertexSpecular( (g_pBasePassLightPosition) , (g_cBasePassLightColor) );
		PixelShader  = compile ps_3_0 PS_DiffuseVertexSpecular( (g_cBasePassLightColor), (g_cMaterialDiffuse), (g_cMaterialAmbient) );
    }
    pass PerLightPass
	<
	// tell the mad3D engine pass to call once for every light while changing the value 
	// of the g_vLightColor & g_vLightPosition parameters
	bool bPerLightPass = true;
	>
	{
    	AlphaBlendEnable = true;
		ZFunc = LESSEQUAL;
		ZEnable = true;
		SrcBlend = one;
		DestBlend = one;
		VertexShader = compile vs_3_0 VS_DiffuseVertexSpecular( (g_pPerLightPassLightPosition) , (g_cPerLightPassLightColor) );
		PixelShader  = compile ps_3_0 PS_DiffuseVertexSpecular( (g_cBasePassLightColor), (g_cMaterialDiffuse), float3(0,0,0) );
    }
}

///////////////////////////////////////////////////////////////////////////////

// Default lighting.
#include "FixedPipelineDiffuse.fxh"

///////////////////////////////////////////////////////////////////////////////
