
#define _USE_WVP
#define _USE_W
#define _USE_WInvTrans
#define _USE_SPECMAP
#define _USE_DIFFUSEMAP
#define _USE_DETAILMAP
#define _USE_MATERIAL
#define _USE_LIGHTS

#include "StandardParameters.fxh"
#include "Effect_common.fxh"

//------------------------------------
struct diffuseNormalSpecularVertexOutput 
{
    float4 pPosition	: POSITION;
    float2 tcDiffuse	: TEXCOORD0;
    float2 tcSpecular	: TEXCOORD2;
    float2 tcDetail		: TEXCOORD3;
    float4 vLight		: TEXCOORD4;
    float3 vHalfAngle	: TEXCOORD5;
    float3 vNormal		: TEXCOORD1;
};


diffuseNormalSpecularVertexOutput VS_DiffuseSpecularDetail( vertexInput IN ,
																	uniform float4 vLightPos,
																	uniform float4 vLightColor ) 
{
    diffuseNormalSpecularVertexOutput OUT;

	// Transform position to clip space and output it
    OUT.pPosition = ModelToScreenSpace( IN.pPosition );
    
    // Output texture coordinates.
	OUT.tcDiffuse = IN.tcDiffuse;
	OUT.tcSpecular = IN.tcDiffuse;
	
	// compute and output the detail coordinates
	OUT.tcDetail = DiffuseToDetailCoord( IN.tcDiffuse );
	
	// get world position for diffuse and specular lighting
	float4 pWorldPosition = ModelToWorldSpace( IN.pPosition );
	
	// light intensity and direction from pos and light pos
	OUT.vLight = ComputeLightDirAndIntensityToWorldPoint( vLightPos.xyz , pWorldPosition.xyz, vLightPos.w, vLightColor.w );

	// world normal for dot
	OUT.vNormal = ModelToWorldSpaceRotate( IN.vNormal );
	
	// the half-angle vector 
	OUT.vHalfAngle = ComputeHalfAngle( OUT.vLight.xyz , pWorldPosition );

	return OUT;
}


float4 PS_DiffuseSpecularDetail( diffuseNormalSpecularVertexOutput IN,
											uniform float3 cLightColor,
											uniform float4 cMaterialDiffuse, // we want alpha mat value to carry through
											uniform float3 cMaterialAmbient ): COLOR
{
	// fetch base texture
	float4 cDiffTex =
		tex2D( g_sampDiffuse, IN.tcDiffuse );
	// fetch specular texture
	float4 cSpecTex =
		tex2D( g_sampSpecular, IN.tcSpecular );
	// fetch detail texture9	
	float4 cDetailTex =
		tex2D( g_sampDetail, IN.tcDetail );

	// The effective diffuse texture color
	float4 cDiffMat = DetailColorScale( cDiffTex, cDetailTex );

	// Compute the magnitude of light hitting the surface.
	float fNDotL = saturate( dot( normalize(IN.vNormal.xyz), normalize(IN.vLight.xyz) ) );
	// Compute the amount of specular light reflected towards the camera.
	float fNDotH = saturate( dot( normalize(IN.vNormal.xyz), normalize(IN.vHalfAngle.xyz) ) );
	
	// if diffuse does not light the spot at all specular should not
	if( fNDotL <= 0 )
	{
		fNDotH = 0;
	}
	
	// We use specMag^(shininess) for our fall-off.
	float fNDotHExp = pow( fNDotH, shininess );
	
	// Compute the specular color
	// use white light color, actually looks better
	float3 cSpecular = cSpecTex.xyz * cLightColor * fNDotHExp * IN.vLight.w;
	
	// Compute the diffuse color, witht the per pass light color parameter
	float3 cDiffuse = cMaterialDiffuse.rgb * cLightColor * fNDotL * IN.vLight.w;
	
	// Compute the ambient color, we do not use light ambient so it end up 
	// being only material ambient, default black.
	float3 cAmbient = cMaterialAmbient.xyz;
		
	// combine lighting with diffuse and detail texture
	return float4( saturate(cDiffuse+cAmbient) * cDiffMat + cSpecular , cDiffMat.a * cMaterialDiffuse.a );
}


//-----------------------------------
// Bump mapping with just diffuse information
technique DiffuseBumpSpecularDetail
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1ST";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_DEFAULT;
// the number of lights the effect can do
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
	// pass to use for first light
	pass BasePass
	<
    	// tell the mad3D engine that this is the base pass 
    	// to do once
    	bool bBasePass = true;
	>
    {
		VertexShader = compile vs_3_0 VS_DiffuseSpecularDetail( (g_pBasePassLightPosition) , (g_cBasePassLightColor) );
		PixelShader  = compile ps_3_0 PS_DiffuseSpecularDetail( (g_cBasePassLightColor), (g_cMaterialDiffuse), (g_cMaterialAmbient) );
    }
    
    // pass to use for eah additionnal light
	pass PerLightPass
	<
	// tell the mad3D engine pass to call once for every light while changing the value 
	// of the g_vLightColor & g_vLightPosition parameters
	bool bPerLightPass = true;
	>
    {
    	AlphaBlendEnable = true;
		DestBlend = one;
    	VertexShader = compile vs_3_0 VS_DiffuseSpecularDetail( (g_pPerLightPassLightPosition) , (g_cPerLightPassLightColor) );
		PixelShader  = compile ps_3_0 PS_DiffuseSpecularDetail( (g_cPerLightPassLightColor), (g_cMaterialDiffuse), float3( 0,0,0 ) );
    }
}


///////////////////////////////////////////////////////////////////////////////

// Default lighting.
#include "FixedPipelineDiffuse.fxh"

///////////////////////////////////////////////////////////////////////////////
