///////////////////////////////////////////////////////////////////////////////
// Vertex Shader
///////////////////////////////////////////////////////////////////////////////
#define _USE_WVP
#define _USE_W
#define _USE_WInvTrans
#define _USE_MATERIAL
#define _USE_DIFFUSEMAP
#define _USE_LIGHTS
#define _USE_SPECMAP



#include "StandardParameters.fxh"
#include "effect_common.fxh"



///////////////////////////////////////////////////////////////////////////////
// this value must exactly match the enum value of
// The output of the Vertex Shader
struct VS_OUTPUT
{
  // The transformed position
  float4 vPosition : POSITION;
  // The diffuse color 
  float4 cDiffuse : COLOR0;
  // specular
  float3 cSpecular : COLOR1;
 // The UV coordinates of the texture.
  float2 tcDiffuse : TEXCOORD0;
};

///////////////////////////////////////////////////////////////////////////////

// simplest vertex shader just transforms and ignores normals
VS_OUTPUT VS_TransformedTex(
  const float3 vPosition : POSITION,
  const float3 vNormal : NORMAL,
  const float2 tcDiffuse : TEXCOORD0,
  uniform int iLightCount
  )
{
	VS_OUTPUT vsOut;
	vsOut.vPosition = ModelToScreenSpace( vPosition );
	// get world position for diffuse lighting
	float4 pWorldPosition = ModelToWorldSpace( vPosition );

	// start at black then add the lights
	float3 cDiffuseColor = g_cMaterialAmbient;
	float3 cSpecularColor = 0;
	
	// bring normal to world
	float3 vWorldNormal = ModelToWorldSpaceRotate( vNormal );
	
	// for each light just compute color
	for( int i = 0; i <iLightCount; i++ ) 
	{
		// light intensity and direction from vert pos and light pos
		float4 vLight = ComputeLightDirAndIntensityToWorldPoint( g_pLight[i].xyz , pWorldPosition, g_pLight[i].w, g_cLightDiffuse[i].w );

		// (N.L)
		float fNDotL = max( 0, dot( vWorldNormal, vLight.xyz ) );
	
		// diffuse color
		cDiffuseColor += g_cLightDiffuse[i].rgb * vLight.w * fNDotL * g_cMaterialDiffuse;
		
		// (N.H) specular
		float3 H = ComputeHalfAngle( vLight.xyz , pWorldPosition );
		float fNDotH = max( 0, dot( vWorldNormal, H ) );
		
		// if diffuse does not light the spot at all specular should not
		if( fNDotL <= 0 )
		{
			fNDotH = 0;
		}
	
		// We use specMag^(shininess) for our fall-off.
		float fNDotHExp = pow( fNDotH, shininess );
		cSpecularColor += g_cLightDiffuse[i].rgb * g_cMaterialSpecular * vLight.w * fNDotHExp;
		
	}
	// send alpha down with color
	vsOut.cDiffuse = min( float4(1,1,1,1) , float4( cDiffuseColor , g_cMaterialDiffuse.a ) );
	
	// output specular color
	vsOut.cSpecular = min( float4(1,1,1,1) , cSpecularColor );
	
	// output texture coordinates
	vsOut.tcDiffuse = tcDiffuse;
	return vsOut;
}


///////////////////////////////////////////////////////////////////////////////
// Pixel Shader
///////////////////////////////////////////////////////////////////////////////

// Modulates the diffuse color of the vertex with the diffuse map
float4 PS_Diffuse(VS_OUTPUT psIn,
				  uniform float4 cMaterialDiffuse) : COLOR0
{
	// light the texture and use the material's alpha value
	float4 result = tex2D( g_sampDiffuse, psIn.tcDiffuse );
	
	// mul with tex alpha
	result.a *= psIn.cDiffuse.a;
	
	result.rgb = result.rgb*psIn.cDiffuse.rgb + psIn.cSpecular;
	return result;
}

///////////////////////////////////////////////////////////////////////////////

// renders the object with just a diffuse texture
technique PlanetCapture
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_GOOD;
// the number of lights the effect can do
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
	// render the diffuse texture
	pass DiffusePass
	{
		VertexShader = compile vs_3_0 VS_TransformedTex( 8 );
		PixelShader = compile ps_3_0 PS_Diffuse(g_cMaterialDiffuse);
	}
}
// renders the object with just a diffuse texture
technique PlanetCapture_Low
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_LOW;
// the number of lights the effect can do
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
	// render the diffuse texture
	pass DiffusePass
	{
		VertexShader = compile vs_3_0 VS_TransformedTex( 2 );
		PixelShader = compile ps_3_0 PS_Diffuse(g_cMaterialDiffuse);
	}
}
///////////////////////////////////////////////////////////////////////////////

// Default lighting.
#include "FixedPipelineDiffuse.fxh"

///////////////////////////////////////////////////////////////////////////////

