///////////////////////////////////////////////////////////////////////////////
// Vertex Shader
///////////////////////////////////////////////////////////////////////////////

#define _USE_WVP
#define _USE_W
#define _USE_UVOFFSET
#define _USE_CLOAKING
#define _USE_DIFFUSEMAP
#define _USE_CLIPPLANE
#define _USE_MATERIAL





#include "StandardParameters.fxh"
#include "Effect_common.fxh"

///////////////////////////////////////////////////////////////////////////////
// this value must exactly match the enum value of

// The output of the Vertex Shader
struct VS_OUTPUT
{
  // The transformed position
  float4 vPosition : POSITION;
 // The UV coordinates of the texture.
  float2 vTextureUV : TEXCOORD0;
  float4 vWorldPosition : TEXCOORD1;
};

///////////////////////////////////////////////////////////////////////////////

// simplest vertex shader just transforms and ignores normals
VS_OUTPUT VS_TransformedTex1(
  const float3 vPosition : POSITION,
  const float3 vNormal : NORMAL,
  const float2 vTextureUV : TEXCOORD0
  )
{
	VS_OUTPUT vsOut;
	vsOut.vPosition = ModelToScreenSpace( vPosition );
	vsOut.vWorldPosition = ModelToWorldSpace(vPosition);
	//no lighting in self illumination
	// send diffuse color right tru
	vsOut.vTextureUV = vTextureUV+g_DiffuseUVOffset;
	
	// if cloaking is enabled, this object needs to be distorted so that its depth value is 
    // written correctly into the depth buffer; the cloaking effect expects the depth buffer 
    // to be established before it begins its processing - jmg
    if (g_bEnableCloakDistortion)
    {
    	// distort the model in screen space - jmg
		vsOut.vPosition.xy += ComputeCloakingDistortion(vsOut.vPosition.xy, g_fDistortionWaveAmplitude, g_CloakTime);
	}
		
	return vsOut;
}

///////////////////////////////////////////////////////////////////////////////
// Pixel Shader
///////////////////////////////////////////////////////////////////////////////

// Modulates the diffuse color of the vertex with the diffuse map
float4 PS_SelfIllumination( VS_OUTPUT psIn,
							uniform bool bDoClipping ): COLOR0
{
	float4 color = tex2D( g_sampDiffuse, psIn.vTextureUV );
	
	if( bDoClipping )
	{
		// if the pixel falls behind the specified clipping plane, kill it
		clip(dot(psIn.vWorldPosition - g_vClipPlanePosition, g_vClipPlaneNormal));
	}

	// we want to fade that out with global alpha
	return float4(color.rgb , color.a * g_cMaterialDiffuse.a);
}

///////////////////////////////////////////////////////////////////////////////
// default Effect
///////////////////////////////////////////////////////////////////////////////

// renders the object with just a diffuse texture
technique RenderDiffuse
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_DEFAULT;
// the number of lights the effect can do
// this effect ignores lights so it can render any number ...
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
	// render the diffuse texture
	pass DiffusePass
	{
		AlphaBlendEnable = true;
		SrcBlend = SRCALPHA;
		DestBlend = INVSRCALPHA;
		VertexShader = compile vs_3_0 VS_TransformedTex1( );
		PixelShader = compile ps_3_0 PS_SelfIllumination( true );
	}
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

// do not clip
technique RenderLow
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_LOW;
// the number of lights the effect can do
// this effect ignores lights so it can render any number ...
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
	// render the diffuse texture
	pass DiffusePass
	{
		AlphaBlendEnable = true;
		SrcBlend = SRCALPHA;
		DestBlend = INVSRCALPHA;
		VertexShader = compile vs_3_0 VS_TransformedTex1( );
		PixelShader = compile ps_3_0 PS_SelfIllumination( false );
	}
}
// do nothing
technique RenderFallback
<
// this value must exactly match the enum value of
// Mad3D_VertexFormats::eFormat
string Mad3D_VertexFormat = "kPosNormTex1";
// this is used by the game to choose an effect based on user setting
// lower value = less quality & more speed
int Quality = QUALITY_FALLBACK;
// the number of lights the effect can do
// this effect ignores lights so it can render any number ...
int SupportedLightsMin = 0;
int SupportedLightsMax = 8;
>
{
}
